/*
 *  Released under the GNU LGPL.  See http://www.gnu.org/licenses/lgpl.txt
 *
 *  This program is part of the DeSmet C Compiler
 *
 *  This library is free software * you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundatation * either version 2.1 of the License, or
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY * without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 *  License for more details.
 */
/*
 *	Cos(x)		Abramowitz & Stegun 4.3.45
 *
 *				if tan(x) = a, then cos(x) = 1 / sqrt(1 + a*a)
 *									sin(x) = a / sqrt(1 + a*a)
 *
 *				Abramowitz & Stegun 4.3.17
 *
 *				cos( z1 + z2 ) = cos(z1)*cos(z2) - sin(z1)*sin(z2)
 */


static char twopi[8]={0x18,0x2d,0x44,0x54,0xfb,0x21,0x19,0x40};
static char piover4[8]={0x18,0x2d,0x44,0x54,0xfb,0x21,0xe9,0x3f};
struct {double dbl;};
#define	TWO_PI		(twopi->dbl)	/* 6.283185307 */
#define PI_OVER_4	(piover4->dbl)	/* 0.785398164 */

double _tan();
double sqrt();
double fabs();

static double sintab[] =
	{
	 0.0,
	 0.7071067811,
	 1.0,
	 0.7071067811,
	 0.0,
	-0.7071067811,
	-1.0,
	-0.7071067811,
	};

static double costab[] =
	{
	 1.0,
	 0.7071067811,
	 0.0,
	-0.7071067811,
	-1.0,
	-0.7071067811,
	 0.0,
	 0.7071067811,
	};

double cos(x)
double x;
	{
	double a,t;
	int i;

	x = fabs(x);			/* cos(-z) = cos(z) A&S 4.3.14 */
	while ( x > TWO_PI )
		x -= TWO_PI;		/* reduce to 0 <= x <= 2PI */
	i = 0;
	while ( x > PI_OVER_4 )
		{
		x -= PI_OVER_4;		/* reduce to 0 <= x <= PI/4 */
		i++;				/* octant */
		}
	a = _tan(x);			/* ASSERT: 0.0 <= a <= 1.0 */
	t = 1.0 + a * a;
	t = sqrt( t );
	return( costab[i] * ( 1.0 / t ) - sintab[i] * ( a / t ) );
	}
