/*
 *  Released under the GNU LGPL.  See http://www.gnu.org/licenses/lgpl.txt
 *
 *  This program is part of the DeSmet C Compiler
 *
 *  This library is free software * you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundatation * either version 2.1 of the License, or
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY * without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 *  License for more details.
 */
/*	MSDOS-II only version of File I/O. */

/*	The tradeoffs in using this version instead of the normal library I/O are:
	. This version only works with MS-DOS V2.0.
	. This version is very slow with getc/putc I/O or with small reads
	  or writes.
	. This version supports 32 files instead of 8.
	. This version is several K smaller.
	. The open( ,3) (look for a control Z) option is not supported.
	. This version can be changed.

	To use, compile and include in the BIND list, e.g.
	A>C88 CREAT2
	A>BIND (normal bind list) CREAT2

*/


#define CONTC	3
#define BS		8
#define LF		10
#define CR		13
#define CONTX	24
#define CONTZ	26
#define RUB		127


#define STDPRN	4
#define	NFILE	32

static int  files[NFILE]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
				   -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
int _oldfile=99;		/* last file scanf'ed	*/
int _ungotten[NFILE]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
					  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
					  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};




/*	CREAT			filenum=creat(name)		*/

creat(name)
	char *name; {
	int  fnum,err;
	char newname[65];

	_fixname(newname,name);
	if ((fnum=_newfile()) == -1) return -1;
	err=files[fnum]=_creat(newname);
	return (err == -1) ? -1: fnum | 0x800;
	}





/*	OPEN			filenum=open(name,mode)		*/

open(name,mod)
	char *name,mod; {
	int  fnum,err;
	char newname[65];
	if (mod > 2) mod-=3;	/* mode 3 means test for control Z */
							/* not done with this version */

	if (mod > 2) return -1;
	_fixname(newname,name);
	if ((fnum=_newfile()) == -1) return -1;
	err=files[fnum]=_open(newname,mod);
	return (err == -1) ? -1: fnum | 0x800;
	}


/*		_newfile allocates a unused file number. */

_newfile() {
	int  i;

	for (i=STDPRN+1; i < NFILE; i++) {
		if (files[i] == -1) return i;
		}
	return -1;
	}



/*		_fixname removes the colon from a device name.	*/

_fixname(newname,name)
	char *newname,*name; {

	do {
		*newname++=*name;
		}
	while (*name++);
	if (*(newname-2) == ':') *(newname-2)=0; /* chop off the trailing colon */
	}








/*	READ			charactors read = read(filenum,buffer,nbytes);	*/

read(filenum,buffer,nbytes)
	int  filenum;
	unsigned nbytes;
	char *buffer; {

	filenum &=0x7ff;
	if (filenum > STDPRN) filenum=files[filenum];
	return _read(filenum,buffer,nbytes);
	}



/*	WRITE			charactors written = write(filenum,buffer,nbytes);	*/

write(filenum,buffer,nbytes)
	int  filenum;
	unsigned nbytes;
	char *buffer; {

	filenum &=0x7ff;
	if (filenum > STDPRN) filenum=files[filenum];

	return _write(filenum,buffer,nbytes);
	}



/*	CLOSE			error = close(file number);	*/

close(filenum)
	int  filenum; {

	_oldfile=99;
	filenum &=0x7ff;
	if (filenum > STDPRN) {
		filenum=files[filenum];
		files[filenum]=-1;
		return _close(filenum);
		}
	}


/*	GETC, GETW, PUTC, PUTW		*/

getc(filenum)
	int  filenum; {
	int  value;

	value=0;
	filenum &=0x7ff;
	if (filenum > STDPRN) filenum=files[filenum];
	if (_ungotten[filenum] != -1) {
		value=_ungotten[filenum];
		_ungotten[filenum]=-1;
		return value;
		}
	if (_read(filenum,&value,1) == 0) value=26;
	if (value == 26) value=-1;
	return value;
	}


getw(filenum)
	int  filenum; {
	int  value;

	filenum &=0x7ff;
	if (filenum > STDPRN) filenum=files[filenum];
	if (_read(filenum,&value,2) != 2) value=-1;
	return value;
	}


putc(ch,filenum)
	int  filenum;
	char ch; {
	struct {char byt;};

	return write(filenum,&ch,1);
	}

putw(wrd,filenum)
	int  filenum;
	int  wrd; {

	return write(filenum,&wrd,2);
	}




/*	FGETS  --	INPUT A LINE FROM A FILE	*/

fgets(str,n,filenum)
	char *str;
	int  n,filenum; {
	int  ch,oldstr;

	filenum &=0x7ff;
	if (filenum > STDPRN) filenum=files[filenum];
	oldstr=str;
	while (--n && (ch=getc(filenum)) >= 0 && ch != CONTZ) {
		if (ch != CR) *str++=ch;
		if (ch == '\n') break;
		}
	*str=0;
	if (oldstr == str) oldstr=0;
	return oldstr;
	}



/*	FPUTS  --	WRITE A STRING TO A FILE	*/

fputs(str,filenum)
	char *str;
	int  filenum; {
	int  err;
	char ch;

	filenum &=0x7ff;
	if (filenum > STDPRN) filenum=files[filenum];
	while (ch=*str++) {
		if (ch == '\n') putc('\r',filenum);
		err=putc(ch,filenum);
		}
	return err;
	}



static int getat;

/*	GETS			string address=gets(string address);	*/

gets(str)
	char *str; {

	_gets(getat=str,512);
	return getat;
	}


/*	_gets	gets with a maximum length	*/

_gets(str,max)
	char str[1];
	int  max; {
	int  numin=0,i;
	char ch;

	numin=fgets(str,max,0);
	if (numin) {
		while (*str) {
			if (*str == '\n') *str=0;		
			else str++;
			}
		}
	else getat=0;
	return numin;
	}


/*	UNLINK			error code=unlink(file)		*/

unlink(name)
	char *name; {

	return _unlink(name);
	}

/*	RENAME			error code = rename(old name, new name);	*/

rename(old,new)
	char *old,*new; {

	return _rename(old,new);
	}


long _lseek();


/*	LSEEK			error code = lseek(file number, fileoff, code);
					0=seek from start.
					1=seek from current.
					2=seek from end.			*/

long lseek(filenum,fileoff,code)
	int  filenum;
	long fileoff;
	char code; {

	_oldfile=99;
	filenum&=0x7ff;
	if (filenum > STDPRN) filenum=files[filenum];
	return _lseek(filenum,fileoff,code);
	}


/*	UNGETC  --	save a character for next getc call.
				character = ungetc(character, file number); 		*/

ungetc(c,filenum)
	int c;
	int filenum; {

	filenum &=0x7ff;
	if (_ungotten[filenum] != -1) c=-1;
	_ungotten[filenum]=c;
	return c;
	}
