newlib.c


chdir()

    synopsis:   int chdir(pathname)
                char *pathname;

    example:    int  error;
                error = chdir("c:\c88");

chdir() is the MS-DOS command to change default directory. It returns
0 if sucessful and -1 if it fails. The directory must exist.


chmod()

    synopsis:   int chmod(pathname,newmode);
                char *pathname;
                int  newmode;
    example:    #include <DOS.H>
                int  error;
                error = chmod("myfile",CHDIR_READONLY);

chmod() allows a file to be made write protected or hidden or both.
The constants CHDIR_READONLY and CHDIR_HIDDEN will make a file
readable only or hidden. Use CHDIR_READONLY+CHDIR_HIDDEN to make
a file both read only and hidden. Use a nowmode of 0 to make a
file writable and not hidden. The error return is -1 and means
the file could not be found.



closeall()

    synopsis:  int closeall();
    example:   int error;
               error = closeall();

closeall() flushes all buffers and closes all files. -1 is returned if
an error is detected closing any file.



dup()   dup2()

    synopsis:   int dup(file);
                int dup2(file1,file2);
                int file,file1,file2;
    example:    int fil1,fil2,error;
                fil2 = dup(fil1);
                error = dup2(fil1,fil2);

dup() and dup2() supply a second file number for an open file. dup()
returns the new file number or -1 on error. dup2() allows the new
file number to be specified. If the file2 is open, it will be closed.
dup2() returns -1 on error.



filelength()


    synopsis:  int file;
               long filelength(file);
    example:   int fil;
               long filelen;
               filelen=filelength(fil);


filelength() returns the current length of a file. -1 is returned on error.



findfile()

    synopsis:  char *envname,*filename,*targetbuf;
               int  findfile(envname,filename,targetbuf);
    example:   char target[65];
               if (findfile("PATH","SEE.MAC",target)) ....

findfile() uses an environment variable (PATH or a SET variable list)
to search for a pathname. The default directory is searched first, then
the environment is searched for a match of the envname. If a match is
found, it is assumed the name is set to a semi-colon separated list
of directory names. If a match is found, the target is set to the matching
name and 1 is returned. If no match is found, 0 is returned.


fileno()

    synopsis:  FILE *stream;
               int fileno(stream);
    example:   #include <stdio.h>
               FILE *sfile;
               int  file;
               file=fileno(sfile);

fileno() returns the file number associated with a stream. The result is
undefined if the stream is not open.



FP_OFF() and FP_SEG()

    synopsis:  unsigned FP_OFF(pointer);
               unsigned FP_SEG(pointer);
               char *pointer;
    example:   #include <DOS.H>
               unsigned seg,off;
               char *ptr;
               off = FP_OFF(ptr);
               seg = FP_SEG(ptr);

FP_OFF and FP_SEG are macros that return the offset and segment part
of a pointer. They only work in large case. In small case a pointer is
only an offset.


freopen()

    synopsis:  FILE *freopen(filename,type,stream)
               char *filename,*type;
               FILE *stream;
    example:   #include <stdio.h>
               FILE *nstream;
               nstream = freopen("C:DISKF","w",STDOUT);

freopen() is used to close a stream and redirect it to a new file. It
is normally used to redirect STDIN, STDOUT, STDERR and STDPRN. NULL
is returned on error. The open type is the same as with fopen()-
"r" for read, "w" for write and "a" for append.



ftell()

    synopsis: long ftell(stream)
              FILE *stream;
    example:  #include <stdio.h>
              long ftell();
              FILE *sfile;
              long at;
              at = ftell(sfile);

ftell() returns the current location for a stream.



getdir()

    synopsis: char *getdir(drive,pathbuffer)
              char pathbuffer[65];
    example:  char pathbuf[65];
              int  error;
              error = getdir(0,pathbuf);

getdir() returns the full name of the current directory. The first
argument is the drive number: 0=default, 1=A: etc. The buffer for the
returned name should be 65 characters long. -1 is returned on error.



getenv()

    synopsis:  int getenv(name,buffer,max_length)
               char *name,buffer[80];
               int  max_length;
    example:   char ebuf[80];
               int  error;
               error = getenv("INCLUDE",ebuf,80);

getenv() returns the value associated with an environment variable.
PATH and INCLUDE are examples of environment variables. See the MS-
DOS manual SET command for more info. The return value is the length
of the returned string or -1 if no match is found.



isatty()

    synopsis: int  isatty(file)
              int  file;
    example:  int is_tty,file;
              is_tty = isatty(file);

isatty() returns 1 (TRUE) is the file is a character device - console, printer
etc. and returns 0 (FALSE) if the file is a disk file.




labs()

    synopsis:  long labs(longint)
               long longint;
    example:   long labs(),l;
               l=labs(-1234L);

labs() returns the absolute value of a long integer.



locking()

    synopsis:  int locking(file,mode,numbytes)
               int file,mode,numbytes;
    example:   #include <DOS.H>
               int fil,error;
               error = locking(fil,LOCK,100);

locking() locks a portion of an open file. The example would lock 100 bytes
starting from the current location. locking() only works with MS-DOS V3.xx
or later. It is ignored with earlier releases. The locked portion of the file
cannot be used by another program until locking is called again with a mode
of UNLCK. The UNLCK call must be made with the same file location and
numbytes as the LOCK.

Mode must be one of the following:
          LOCK   lock the bytes.
          UNLCK  unlock the bytes.

locking() returns 0 if sucessful and -1 on error.


mkdir()

    synopsis:  int mkdir(pathname)
               char *pathname;
    example:   int error;
               error = mkdir("C88");

mkdir() creates a new directory in the defauld directory. A value of 0 is
returned if the directory is created. -1 is returned on error.


open()

    synopsis:  int open(pathname,mode)
               char *pathname;
               int mode;
    example:   #include <DOS.H>
               int fil;
               fil = sopen("MYFILE",READ+DENYNONE);

open() makes an existing file available for subsequent read(), write(),
and lseek() calls. With any version of MS-DOS, the mode can be:

           0    READ         read only
           1    WRITE        write only
           2    READWRITE    read and write access

With MS-DOS V3.xx a sharing mode may be added to the mode flag. With no or
COMPAT sharing mode, sharing is allowed with other COMPAT opens. To deny
others read or write access to the file, some other mode should be used.

The share mode must be:
 COMPAT	0x00	/* compatibility mode */
 DENYRW	0x10	/* deny read/write mode */
 DENYWR	0x20	/* deny write mode */
 DENYRD	0x30	/* deny read mode */
 DENYNO	0x40	/* deny none mode */

open() returns the file number or -1 if the open could not be performed.
See the Dos Operating System Version 3.00 Technical Reference for further
details on sharing modes.


putenv()

    synopsis:  int putenv(name,buffer);
    synopsis:  char *name,buffer[80];
    example:   char ebuf[80];
               int  error;
               error = putenv("INCLUDE",ebuf);

putenv() changes the value associated with an environment variable.
PATH and INCLUDE are examples of environment variables. See the MS-
DOS manual SET command for more info. The return value is 0 on
success and -1 on failure.



rmdir()

    synopsis:  int rmdir(pathname)
               char *pathname;
    example:   rmdir("C88");

rmdir() deletes the named directory. The directory mut be empty and
cannot be the current directory. rmdir() returns 0 on success and -1 on
error.



strcmpi()

    synopsis:  int strcmpi(string1,string2)
               char *string1,*string2;
    example:   int eq;
               eq = strcmpi(str,"HELLO");

strcmpi() is the same as strcmp() except that case is ignored by the
comparison. The return values are:
         1    string1 is greater
         0    the strings are the same
         -1   string2 is greater



strcspn()

    synopsis:  int strcspn(string1,string2)
               char *string1,*string2;
    example:   int val;
               val = strcspn("HELLO THERE","AEIOU");

strcspn() looks for any of the characters of string2 in string 1.
It returns the number of characters of string1 that are NOT
in string 2. The example would return 7.



strdup()

    synopsis:  char *strdup(string)
               char *string;
    example:   char *ptr,*strdup();
               ptr = strdup("COPY THIS STRING");

strdup() uses malloc to create a copy of a string. It returns a pointer
to the new string or NULL if the memory could not be allocated.



strlwr()

    synopsis:  char *strlwr(string)
               char *string;
    example:   char x[22];
               strlwr(x);

strlwr() converts any upper case letters in the string to lower case.
It returns a pointer to the string.



strpbrk()

    synopsis:  char *strpbrk(string1,string2)
               char *string1,*string2;
    example:   char *ptr,*strpbrk();
               char *at;
               at = strpbrk("HELLO WORLD",AEIOU");

strpbrk() returns a pointer to the first character in string1 that is
also in string 2. The example would return a pointer to "ELLO". NULL
is returned if there is no match.


strrev()

    synopsis:  char *strrev(string)
               char *string;
    example:   char str[22],*strrev();
               strrev(str);

strrev() reverses the characters in a string. It returns a pointer to the
string.



strset()

    synopsis:  char *strset(string,character)
               char *string;
               char *character;
    example:   char str[22],*strset();
               strset(str,'X');

strset() sets all the characters in a string to the designated character.
It returns a pointer to the string.


strspn()

    synopsis:  int strspn(string1,string2)
               char *string1,string2;
    example:   int val;
               val = strspn("HELLO WORLD","HELP");

sprspn() returns the index of the first character of string1 that is not
in string2. strlen() is returned if all characters match. The example
would return 4.



strtok()

    synopsis:  char *strtok(string,delimiters)
               char *string,*delimiters;
    example:   char *tptr,*strtok();
               tptr=strtok("   , aaa , bbb"," ,\t");

strtok() returns a pointer to the next token in the string. After the
first call, the string argument shouild be replaced with (char *) 0
to indicate that the scanning should continue from the last call.
The second string is a list of delimiters. The returned tokens are
null terminated. strtok() returns a NULL when there are no more
tokens.



strupr()

    synopsis:  char *strupr(string)
               char *string;
    example:   char x[22];
               strupr(x);

strupr() converts any lower case letters in the string to upper case.
It returns a pointer to the string.




