/*
 *  Released under the GNU LGPL.  See http://www.gnu.org/licenses/lgpl.txt
 *
 *  This program is part of the DeSmet C Compiler
 *
 *  This library is free software * you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundatation * either version 2.1 of the License, or
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY * without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 *  License for more details.
 */
/*	OVERLAY.C	--	Load an Overlay.	*/

	/*	header for overlays. at start of .OV file. followed by code,data
		for each overlay except 0 (root).	*/
	/*	ovlen is length of data, code and reserved for each overlay.	*/

#define MAXOVER	40
	static unsigned ovlen[MAXOVER][3];
	static int  fid=-1;
	char num_overlay;
	extern char _msdos2;

/*	OVERLAY_INIT	--	Initilize for Overlays.	*/
/*						The name is the name of the overlay file.	*/

static int findfile(),environ();
overlay_init(name)
	char *name; {

	if ((fid=findfile(name)) == -1) return -1;
	return read(fid,ovlen,sizeof(ovlen)) == sizeof(ovlen) ? 1: -1;
	}


/*	OVERLAY	--	load in the overlay	*/

overlay(ov)
	int  ov; {
	int  i;
	unsigned dataat,codeat,buflen,need,nread,*ptr,bufferat;
	long seekto;
	char buffer[128];

	if (ov == num_overlay) return 1;
	if (ov <= 0 || ov >= MAXOVER || fid == -1 ||
		ovlen[ov][0]+ovlen[ov][1]+ovlen[ov][2] == 0) return -1;

	/*	get code and data base addresses	*/

	dataat=ovlen[0][0]+ovlen[0][2];	/*	base of overlay daya	*/
	codeat=ovlen[0][1];				/* base of overlay code	*/
	seekto=sizeof(ovlen);
	for (i=1; i < ov; i++) {
		seekto+=ovlen[i][0];
		seekto+=ovlen[i][1];
		}
	lseek(fid,seekto,0);
	if (ovlen[ov][1]) {		/* read in the code	*/
		bufferat=dataat;
		buflen=_memory()-dataat;
		if (buflen > 512) buflen&=0xfe00;	/* read 512's	*/
		else if (buflen < 128) {
			bufferat=buffer;
			buflen=sizeof(buffer);
			}

		need=ovlen[ov][1];
		do {
			nread=need > buflen ? buflen: need;
			if (read(fid,bufferat,nread) != nread) return -1;
			_lmove(nread,bufferat,_showds(),codeat,_showcs());
			codeat+=nread;
			need-=nread;
			}
		while (need);
		}

	if (ovlen[ov][0]) {		/* read in the data	*/
		if (read(fid,dataat,ovlen[ov][0]) != ovlen[ov][0]) return -1;
		}

	if (ovlen[ov][2]) {		/* clear reserved	*/
		ptr=dataat+ovlen[ov][0];
		i=(ovlen[ov][2]+1)>>1;
		while (i--) *ptr=0;
		}
	num_overlay=ov;
	return 1;
	}

/*	OVERLAY_CLOSE  --	close the overlay file.		*/

overlay_close() {
	int temp_fid;

	temp_fid=fid;
	fid=-1;
	return close(temp_fid);
	}




/*	FINDFILE.C	*/
/*
	This file contains the routine to locate a file, utilizing the PATH
	environment variable for the directories to search.

	Interface:

		findfile(filename)

		where filename is the name of the file to be searched for,
		      returns file pointer or -1.

	This program uses the environ routines to access the PATH variable.
*/

findfile(filename)
	char *filename; {
	char target_buf[65];
	int fid,i;
	char paths[256], *p_ptr, *t_ptr,gotdot;

	p_ptr=filename;
	gotdot=0;
	while (*p_ptr) if (*p_ptr++ == '.') gotdot=1;	

	/* first check in the local directory */
	strcpy(target_buf, filename);
	if (gotdot == 0) strcat(target_buf,".OV");
	if ((fid = open(target_buf, 0)) != -1 || _msdos2 == 0) return fid;
	fid = environ("PATH", paths, 256);
	p_ptr = paths;

	while (*p_ptr != 0) {
		/* copy the directory name */
		t_ptr = target_buf;
		while (*p_ptr != ';' && *p_ptr != 0) {
			*t_ptr++ = *p_ptr++;
			}
		if (*(t_ptr-1) != '/' && *(t_ptr-1) != '\\') *t_ptr++ = '\\';
		*t_ptr = 0;
		if (*p_ptr) p_ptr++;		/* beyond the ';' */
		strcat(target_buf, filename);
		if (gotdot == 0) strcat(target_buf,".OV");
		if ((fid = open(target_buf, 0)) != -1) return fid;
		fid = open(target_buf, 0);
		}
	return (-1);							/* can't find one */
	}



/*  ENVIRON.C	*/

/*
	This file contains the routine for searching the environment
	area for a given string.

	Interface:

		environ(search_str, buffer, buf_len)

		where search_str is the actual string being searched for,
			  buffer is the location to store the string if found,
			  buf_len is the maximum number of characters to store
			          into the buffer (includes terminator).

        -1 is returned if the environment variable isn't found,
        otherwise the length of the string copied is returned.
*/

extern unsigned _pcb;		/* c88 variable with the PSP segment */

int environ(search_str, buffer, buf_len)
	char *search_str, *buffer; int buf_len; {

#asm

	mov		ax, _pcb_;				; get the env segment into ES
	mov		es, ax					; 
	mov		ax, es:[2CH]
	mov		es,ax

	mov		bx, [6][bp]				; buffer
	mov		byte[bx],0
	mov		si, 0					; offset into the environment
	jmp		outer_begin

outer_loop:
	inc		si
outer_begin:
	mov		di, [4][bp]				; search_str
	cmp		byte es:[si],0
	jne		cmp_loop
									; not found
	mov		ax,-1
	pop		bp
	ret

cmp_loop:
	mov		al, byte es:[si]		; pick up the search character
	cmp		al,'='
	je		end_str
	cmp		al, byte[di]
	jne		outer_next
	inc		si
	inc		di
	jmp		cmp_loop

end_str:
	cmp		byte[di], 0
	jne		outer_next
									; got it, copy them bytes!
	mov		cx, [8][bp]				; buf_len
	inc		si						; beyond the '='

move_loop:
	mov		al, byte es:[si]
	or		al,al
	jz		now_return
	dec		cx
	jz		now_return
	mov		byte[bx],al
	inc		si
	inc		bx
	jmp		move_loop

now_return:
	mov		byte[bx],0
	mov		ax, [8][bp]
	sub		ax, cx
	pop		bp
	ret

outer_next:
	cmp		byte es:[si],0				; skip to the next entry
	je		outer_loop
	inc		si
	jmp		outer_next
#
}
